/*******************************************************************************
 * Copyright (c) 2011, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/

/* $Id: NLObjectSingleton.m 248 2010-08-25 14:12:28Z  $ */

#import "NLObjectSingleton.h"

@implementation NLObjectSingleton

static NSMutableDictionary * _instances;

+ ( id )getInstance
{
    return [ self sharedInstance ];
}

+ ( id )sharedInstance
{
    @synchronized( self )
    {
        if( [ _instances objectForKey: NSStringFromClass( [ self class ] ) ] == nil )
        {
            [ [ [ self alloc ] init ] csInit ];
        }
    }
    
    return [ _instances objectForKey: NSStringFromClass( [ self class ] ) ];
}

+ ( id )allocWithZone:( NSZone * )zone 
{
    @synchronized( self )
    {
        if( _instances == nil )
        {
            _instances = [ [ NSMutableDictionary dictionaryWithCapacity: 100 ] retain ];
        }
        
        if( [ _instances objectForKey: NSStringFromClass( [ self class ] ) ] == nil )
        {
            [ _instances setObject: [ super allocWithZone: zone ] forKey: NSStringFromClass( [ self class ] ) ];
        }
        
        return [ _instances objectForKey: NSStringFromClass( [ self class ] ) ];
    }
    
    return nil;
}

- ( id )copyWithZone:( NSZone * )zone 
{
    ( void )zone;
    
    return self;
}

- ( id )retain
{
    return self;
}

- ( NSUInteger )retainCount
{
    return UINT_MAX;
}

- ( oneway void )release
{}

- ( id )autorelease
{
    return self;
}

- ( BOOL )isInited
{
    return NL_inited;
}

- ( id )csInit
{
    NL_inited = YES;
    
    return self;
}

@end
