/*******************************************************************************
 * Copyright (c) 2011, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/

/* $Id: NLLogMessage.h 245 2010-05-30 14:18:05Z  $ */

/*!
 * @header
 * 
 * @discussion  This file should not be included directly by application source
 *              files using the NutShell framework. Including 'NutShell.h' is
 *              preferred over importing individual files because it will use a
 *              precompiled version.
 */

// Local includes
#import "NLObject.h"

/*!
 * @enum        MessageTypes
 * 
 * @abstract    The types of the log messages
 */
enum
{
    NLMessageLog   = 0,
    NLMessageDebug = 1
};

/*!
 * @abstract    Log message object
 * 
 * @version     1.0
 * @namespace   NutShell
 */
@interface NLLogMessage: NLObject
{
@protected
    
    /*!
     * @abstract    The message
     */
    NSString * message;
    
    /*!
     * @abstract    The message's date
     */
    NSDate * date;
    
    /*!
     * @abstract    The message's type
     */
    int type;
    
@private
    
    /*!
     * @abstract    Reserved 1
     * 
     * @discussion  Reserved instance variables to help ensure binary
     *              compatibility with future versions of the class.
     */
    id NL_r1 __attribute__( ( unused ) );
    
    /*!
     * @abstract    Reserved 2
     * 
     * @discussion  Reserved instance variables to help ensure binary
     *              compatibility with future versions of the class.
     */
    id NL_r2 __attribute__( ( unused ) );
}

/*! @property */
@property( copy, readwrite ) NSString * message;

/*! @property */
@property( copy, readwrite ) NSDate * date;

/*! @property */
@property( assign, readwrite ) int type;

/*!
 * @abstract    Creates a new log message with a specific message text
 * 
 * @param       str         The message text
 * @return      The instance of the message object
 */
+ ( id )messageWithMessage: ( NSString * )str;

/*!
 * @abstract    Creates a new log message with a specific message text and date
 * 
 * @param       str         The message text
 * @param       time        The message date
 * @return      The instance of the message object
 */
+ ( id )messageWithMessage: ( NSString * )str date: ( NSDate * )messageTime;

/*!
 * @abstract    Creates a new log message with a specific message text, date and type
 * 
 * @param       str         The message text
 * @param       time        The message date
 * @param       messageType The message type
 * @return      The instance of the message object
 */
+ ( id )messageWithMessage: ( NSString * )str date: ( NSDate * )messageTime type: ( int )messageType;

/*!
 * @abstract    Initializes a newly allocated log message with a specific message text
 * 
 * @param       str         The message text
 * @return      The instance of the message object
 */
- ( id )initWithMessage: ( NSString * )str;

/*!
 * @abstract    Initializes a newly allocated log message with a specific message text and date
 * 
 * @param       str         The message text
 * @param       time        The message date
 * @return      The instance of the message object
 */
- ( id )initWithMessage: ( NSString * )str date: ( NSDate * )messageTime;

/*!
 * @abstract    Initializes a newly allocated log message with a specific message text, date and type
 * 
 * @param       str         The message text
 * @param       time        The message date
 * @param       messageType The message type
 * @return      The instance of the message object
 */
- ( id )initWithMessage: ( NSString * )str date: ( NSDate * )messageTime type: ( int )messageType;

@end
